package handlers_test

import (
	"encoding/base64"
	"net/http"
	"testing"

	"github.com/Masterminds/squirrel"
	"github.com/stretchr/testify/assert"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestAccountTemplateUpdate(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx := t.Context()
	ctx = log.WithContext(ctx)

	tester := apitester.NewAPITester(ctx, t, nil)
	tmpl := models.Template{
		Account:    "janedoe",
		Name:       "hello",
		Language:   "mustache",
		Produces:   "text/plain",
		BodyFormat: "text",
		Body:       "Hello {{name}}!",
	}

	defer tester.Close()
	defer tester.Logout()
	tester.Login("admin", "admin")

	tester.CreateUser("janedoe", "janedoe")
	tester.Login("janedoe", "janedoe")

	t.Run("Update non-existent template", func(t *testing.T) {
		r := tester.APITest("template_update").
			Put("/api/v1/template/janedoe/hello").
			JSON(&models.Template{}).
			Expect(t).
			Status(http.StatusNotFound).
			End()

		var httpErr models.Error

		r.JSON(&httpErr)
		assert.Equal(t, "No such template", httpErr.Message)
	})

	t.Run("Update body", func(t *testing.T) {
		defer func() {
			_, _ = database.Exec(tester.DB, squirrel.Delete(models.TemplateDBTable), &log)
		}()

		var responseTemplate models.Template

		tester.CreateTemplate(
			"janedoe",
			&tmpl,
			&responseTemplate,
		)

		responseTemplate.Body = "Hi {{name}}!"
		r := tester.APITest("template_update").
			Put("/api/v1/template/janedoe/hello").
			JSON(&responseTemplate).
			Expect(t).
			Status(http.StatusOK).
			End()
		r.JSON(&responseTemplate)
		assert.Equal(t,
			base64.StdEncoding.EncodeToString([]byte("Hi {{name}}!")), responseTemplate.Body)
	})

	t.Run("Rename", func(t *testing.T) {
		defer func() {
			_, _ = database.Exec(tester.DB, squirrel.Delete(models.TemplateDBTable), &log)
		}()

		var responseTemplate models.Template

		tester.CreateTemplate(
			"janedoe",
			&tmpl,
			&responseTemplate,
		)

		responseTemplate.Name = "hi"
		r := tester.APITest("template_update").
			Put("/api/v1/template/janedoe/hello").
			JSON(&responseTemplate).
			Expect(t).
			Status(http.StatusOK).
			End()
		r.JSON(&responseTemplate)
		assert.Equal(t, "hi", responseTemplate.Name)
	})
}
