package handlers_test

import (
	"encoding/json"
	"fmt"
	"strconv"
	"testing"
	"time"

	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestRenderLogs(t *testing.T) {
	ctx := t.Context()

	tester := apitester.NewAPITester(ctx, t, nil)
	defer tester.Close()

	now := time.Now()
	tester.SetNow(now)
	// db := database.NewSQLHelper(ctx, tester.DB, tester.Logger)
	// create user johndoe
	tester.Login("admin", "admin")
	tester.CreateUser("johndoe", "johndoe")
	tester.Logout()
	tester.Login("johndoe", "johndoe")

	// get engine list
	// engines := tester.ListEngines()

	// tester.Logger.Info().Msgf("engines: %s", engines)
	// require.Equal(t, http.StatusOK, code)

	// add a template to its name
	var createdTemplate models.Template

	tester.CreateTemplate(
		"johndoe",
		&models.Template{
			Name:       "template1",
			Body:       "bonjour {{ user }}",
			BodyFormat: models.BodyFormatText,
			Language:   "mustache",
			Produces:   "text/html",
		},
		&createdTemplate,
	)

	b, err := json.Marshal(&createdTemplate)
	require.NoError(t, err)
	tester.Logger.Info().Msgf("created template %s", string(b))

	var response []*models.Document

	tester.Render(
		&models.RenderRequest{
			Accept: "text/html",
			Template: &models.Template{
				Account: "johndoe",
				Name:    "template1",
			},
			Data: models.Dataset{
				apitester.JSONObj{"user": "janedoe"},
			},
		},
		&response,
	)
	require.Len(t, response, 1)
	s, err := response[0].String()
	require.NoError(t, err)
	require.Equal(t, "bonjour janedoe", s)

	tester.Logout()
	tester.Login("admin", "admin")

	defer tester.Logout()
	csvFilename := strconv.Itoa(tester.Now().Year()) +
		"-" +
		fmt.Sprintf("%02d", int(tester.Now().Month())) +
		".csv"

	s = tester.RenderLogs(csvFilename)
	tester.Logger.Info().Msgf("got renderlogs:\n%s", s)
}
