package handlers_test

import (
	"net/http"
	"strconv"
	"testing"

	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestUserList(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx := t.Context()
	ctx = log.WithContext(ctx)

	tester := apitester.NewAPITester(ctx, t, nil)
	defer tester.Close()

	// login as admin and create 10 users
	tester.Login("admin", "admin")

	for k := range 10 {
		john := "johndoe_" + strconv.Itoa(k)
		tester.CreateUser(john, "password"+strconv.Itoa(k))
		// adding some dummy activity for the user JohnDoe
		AddUserActivity(t, log, tester.DB, john, 4, 2)
	}

	tester.Logout()
	t.Run("unauthorized", func(t *testing.T) {
		// try and get the list as johndoe_1
		tester.Login("johndoe_1", "password1")
		defer tester.Logout()
		_ = tester.APITest("user_list").
			Get("/api/v1/user").
			Expect(t).
			Status(http.StatusUnauthorized).
			End()
	})
	t.Run("basic", func(t *testing.T) {
		tester.Login("admin", "admin")
		defer tester.Logout()
		r := tester.APITest("user_list").
			Get("/api/v1/user").
			Expect(t).
			Status(http.StatusOK).
			End()

		var users []*models.Userinfo

		r.JSON(&users)
		// 11 users since admin is there as well!
		require.Len(t, users, 11)
	})
}
