package handlers_test

import (
	"net/http"
	"testing"

	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestUserUpdate(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx := t.Context()
	ctx = log.WithContext(ctx)

	tester := apitester.NewAPITester(ctx, t, nil)
	defer tester.Close()

	// login as admin and create a user
	tester.Login("admin", "admin")
	tester.CreateUser("johndoe", "johndoe")
	tester.CreateUser("janedoe", "janedoe")

	// adding some dummy activity for the user JohnDoe
	qtyCurrentMonth := 4
	qtyLastMonth := 2
	AddUserActivity(t, log, tester.DB, "johndoe",
		qtyCurrentMonth, qtyLastMonth)

	r := tester.APITest("user_get").
		Get("/api/v1/user/johndoe").
		Expect(t).
		Status(http.StatusOK).
		End()

	var user models.Userinfo

	r.JSON(&user)

	// Right now any userUpdate with new Email will be discarded.
	newEmail := "johndoe@mynewdomain.com"
	oldEmail := user.Email
	user.Email = newEmail
	// But any other field will be changed by the update
	user.Firstname = "Will"

	tester.Logout()

	t.Run("unauthorized", func(t *testing.T) {
		tester.Login("janedoe", "janedoe")
		defer tester.Logout()
		_ = tester.APITest("user_update").
			Put("/api/v1/user/johndoe").
			JSON(&user).
			Expect(t).
			Status(http.StatusUnauthorized).
			End()
	})
	t.Run("basic", func(t *testing.T) {
		tester.Login("johndoe", "johndoe")
		defer tester.Logout()
		r := tester.APITest("user_update").
			Put("/api/v1/user/johndoe").
			JSON(&user).
			Expect(t).
			Status(http.StatusOK).
			End()

		var updatedUser models.Userinfo

		r.JSON(&updatedUser)
		stats := updatedUser.Activity.RenderStats
		expectedTotal := qtyCurrentMonth + qtyLastMonth
		require.Equal(t, int64(expectedTotal), stats.Total)
		require.Equal(t, int64(qtyCurrentMonth), stats.CurrentMonth)
		require.Equal(t, int64(qtyLastMonth), stats.LastMonth)

		// email hasn't changed
		require.Equal(t, oldEmail, updatedUser.Email)
		// but the firstname has
		require.Equal(t, "Will", updatedUser.Firstname)
	})
}
