package tests

import (
	"testing"

	"github.com/Masterminds/squirrel"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestRegister(t *testing.T) {
	ctx := t.Context()

	tester := apitester.NewAPITester(ctx, t, nil)
	defer tester.Close()

	t.Run("register", func(t *testing.T) {
		defer tester.SetT(t)

		tester.Register(&models.Registration{
			Firstname: "Annie",
			Lastname:  "Ernaux",
			Email:     "example@example.com",
		})

		db := database.NewSQLHelper(ctx, tester.DB, testutils.GetLogger(t))

		var userDB models.UserDB

		require.NoError(t, db.Get(
			&userDB, squirrel.Select("*").From("users").
				Where(squirrel.Eq{"email": "example@example.com"}),
		))

		assert.Equal(t, "example@example.com", userDB.Email)
		assert.Equal(t, "Annie", userDB.Firstname)
		assert.Equal(t, "Ernaux", userDB.Lastname)
		assert.Equal(t, false, userDB.IsAdmin)
	})
}
